@@ -16,7 +16,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <grp.h>
 #include <sys/stat.h>
@@ -455,14 +455,14 @@ perhaps_translate_message (const char *msg)
 /* Internal PAM conversation function. Interfaces between the PAM
  * authentication system and the actual greeter program */
 
-static gint 
-gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
+static int 
+gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
 		     struct pam_response **resp,
 		     void *appdata_ptr)
 {
     int replies = 0;
     int i;
-    char *s;
+    char *s = NULL;
     struct pam_response *reply = NULL;
     const void *p;
     const char *login;
@@ -486,7 +486,7 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
     /* Here we set the login if it wasn't already set,
      * this is kind of anal, but this way we guarantee that
      * the greeter always is up to date on the login */
-    if (pam_get_item (pamh, PAM_USER, &p) == PAM_SUCCESS ||
+    if (pam_get_item (pamh, PAM_USER, (void **)&p) == PAM_SUCCESS ||
 	tmp_PAM_USER != NULL) {
 	    login = (const char *)p;
 
@@ -502,10 +502,10 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
     
     for (replies = 0; replies < num_msg; replies++) {
 	gboolean islogin = FALSE;
-	const char *m = msg[replies]->msg;
+	const char *m = (*msg)[replies].msg;
 	m = perhaps_translate_message (m);
 	
-	switch (msg[replies]->msg_style) {
+	switch ((*msg)[replies].msg_style) {
 	    
 	/* PAM requested textual input with echo on */
 	case PAM_PROMPT_ECHO_ON:
@@ -624,8 +624,8 @@ static struct pam_conv pamc = {
 /* Extra message to give on queries */
 static char *extra_standalone_message = NULL;
 
-static gint 
-gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
+static int 
+gdm_verify_standalone_pam_conv (int num_msg, struct pam_message **msg,
 				struct pam_response **resp,
 				void *appdata_ptr)
 {
@@ -645,9 +645,9 @@ gdm_verify_standalone_pam_conv (int num_msg, const struct pam_message **msg,
 	memset (reply, 0, sizeof (struct pam_response) * num_msg);
 
 	for (replies = 0; replies < num_msg; replies++) {
-		const char *m = msg[replies]->msg;
+		const char *m = (*msg)[replies].msg;
 		m = perhaps_translate_message (m);
-		switch (msg[replies]->msg_style) {
+		switch ((*msg)[replies].msg_style) {
 
 		case PAM_PROMPT_ECHO_ON:
 			if (extra_standalone_message != NULL)
@@ -758,7 +758,7 @@ create_pamh (GdmDisplay *d,
 
 	/* Inform PAM of the user's tty */
 #ifdef	sun
-	if (d->attached == 0)
+	if (d->attached)
 		(void) pam_set_item (pamh, PAM_TTY, "/dev/console");
 	else 
 #endif	/* sun */
@@ -806,6 +806,7 @@ gdm_verify_user (GdmDisplay *d,
     struct passwd *pwent = NULL;
     const void *p;
     char *login, *passreq, *consoleonly;
+    char *pam_stack = NULL;
     int null_tok = 0;
     gboolean credentials_set = FALSE;
     gboolean error_msg_given = FALSE;
@@ -826,6 +827,7 @@ verify_user_again:
 
     /* start the timer for timed logins */
     if ( ! ve_string_empty (gdm_get_value_string (GDM_KEY_TIMED_LOGIN)) &&
+	d->timed_login_ok &&
 	(local || gdm_get_value_bool (GDM_KEY_ALLOW_REMOTE_AUTOLOGIN))) {
 	    gdm_slave_greeter_ctl_no_ret (GDM_STARTTIMER, "");
 	    started_timer = TRUE;
@@ -844,12 +846,21 @@ authenticate_again:
     g_free (tmp_PAM_USER);
     tmp_PAM_USER = g_strdup (login);
 
-    /* Initialize a PAM session for the user */
-    if ( ! create_pamh (d, "gdm", login, &pamc, display, &pamerr)) {
+    /*
+     * Initialize a PAM session for the user...
+     * Get value per-display so different displays can use different
+     * PAM Stacks, in case one display should use a different 
+     * authentication mechanism than another display.
+     */
+    pam_stack = gdm_get_value_string_per_display ((char *)display, GDM_KEY_PAM_STACK);
+
+    if ( ! create_pamh (d, pam_stack, login, &pamc, display, &pamerr)) {
 	    if (started_timer)
 		    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+            g_free (pam_stack);
 	    goto pamerr;
     }
+    g_free (pam_stack);
 
     pam_set_item (pamh, PAM_USER_PROMPT, _("Username:"));
 
@@ -924,7 +935,7 @@ authenticate_again:
     g_free (login);
     login = NULL;
     
-    if ((pamerr = pam_get_item (pamh, PAM_USER, &p)) != PAM_SUCCESS) {
+    if ((pamerr = pam_get_item (pamh, PAM_USER, (void **)&p)) != PAM_SUCCESS) {
 	    login = NULL;
 	    /* is not really an auth problem, but it will
 	       pretty much look as such, it shouldn't really
@@ -1126,9 +1137,7 @@ authenticate_again:
 		    }
 		    if (is_capslock) {
 			    msg = g_strconcat (basemsg, "  ",
-					       _("Please make sure the "
-						 "Caps Lock key is not "
-						 "enabled."),
+					       _("Caps Lock is on."),
 					       NULL);
 		    } else {
 			    msg = g_strdup (basemsg);
@@ -1190,6 +1199,8 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     struct passwd *pwent = NULL;
     const void *p;
     char *passreq;
+    char *pam_stack = NULL;
+    char *pam_service_name = NULL;
     int null_tok = 0;
     gboolean credentials_set;
     const char *after_login;
@@ -1212,11 +1223,23 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 						_("Automatic login"),
 						login);
 
-    /* Initialize a PAM session for the user */
-    if ( ! create_pamh (d, "gdm-autologin", login, &standalone_pamc,
+    /*
+     * Initialize a PAM session for the user...
+     * Get value per-display so different displays can use different
+     * PAM Stacks, in case one display should use a different 
+     * authentication mechanism than another display.
+     */
+    pam_stack = gdm_get_value_string_per_display ((char *)display, GDM_KEY_PAM_STACK);
+    pam_service_name = g_strdup_printf ("%s-autologin", pam_stack);
+
+    if ( ! create_pamh (d, pam_service_name, login, &standalone_pamc,
 			display, &pamerr)) {
+            g_free (pam_stack);
+            g_free (pam_service_name);
 	    goto setup_pamerr;
     }
+    g_free (pam_stack);
+    g_free (pam_service_name);
 
     passreq = gdm_read_default ("PASSREQ=");
     if ((passreq != NULL) &&
@@ -1238,7 +1261,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	    goto setup_pamerr;
     }
 
-    if ((pamerr = pam_get_item (pamh, PAM_USER, &p)) != PAM_SUCCESS) {
+    if ((pamerr = pam_get_item (pamh, PAM_USER, (void **)&p)) != PAM_SUCCESS) {
 	    /* is not really an auth problem, but it will
 	       pretty much look as such, it shouldn't really
 	       happen */
@@ -1327,7 +1350,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     did_setcred = TRUE;
 
 #ifdef sun
-    solaris_xserver_cred (login, d, pwent);
+    solaris_xserver_cred ((char *)login, d, pwent);
 #endif
 
     /* Set credentials */
@@ -1510,13 +1533,14 @@ gdm_verify_check (void)
 {
 	pam_handle_t *ph = NULL;
 
-	if (pam_start ("gdm", NULL, &standalone_pamc, &ph) != PAM_SUCCESS) {
+	if (pam_start (gdm_get_value_string (GDM_KEY_PAM_STACK), NULL,
+		&standalone_pamc, &ph) != PAM_SUCCESS) {
 		ph = NULL; /* be anal */
 
 		closelog ();
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
-        if (gdm_get_value_bool (GDM_KEY_CONSOLE_NOTIFY))
+		if (gdm_get_value_bool (GDM_KEY_CONSOLE_NOTIFY))
 			gdm_text_message_dialog
 				(C_(N_("Can't find PAM configuration for GDM.")));
 		gdm_fail ("gdm_verify_check: %s",
