@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <popt.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <fcntl.h>
@@ -41,10 +40,12 @@
 #include "gdmcommon.h"
 #include "gdmconfig.h"
 
+static gchar **args_remaining;
 static pid_t xnest_pid = 0;
 
 #ifdef ENABLE_IPV6
-static gboolean have_ipv6 () {
+static gboolean
+have_ipv6 () {
 	gint s;
 
 	s = socket (AF_INET6, SOCK_STREAM, 0);
@@ -77,7 +78,7 @@ get_free_display (void)
 	serv_addr.sin_family = AF_INET;
 	serv_addr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
 
-	for (i = 20; i < 3000; i ++) {
+	for (i = 20; i < 3000; i++) {
 		struct stat s;
 		char buf[256];
 		FILE *fp;
@@ -132,13 +133,13 @@ get_free_display (void)
 		}
 
 		g_snprintf (buf, sizeof (buf), "/tmp/.X11-unix/X%d", i);
-		if (stat (buf, &s) == 0 &&
+		if (g_stat (buf, &s) == 0 &&
 		    s.st_uid != getuid ()) {
 			continue;
 		}
 
 		g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
-		if (stat (buf, &s) == 0 &&
+		if (g_stat (buf, &s) == 0 &&
 		    s.st_uid != getuid ()) {
 			continue;
 		}
@@ -163,21 +164,23 @@ static char display_num[BUFSIZ] = "";
 static char indirect_host[BUFSIZ] = "";
 
 /* options for Xnest only mode */
-static const struct poptOption xnest_only_options[] = {
-	{ "xnest", 'x', POPT_ARG_STRING, &xnest_binary, 0, N_("Xnest command line"), N_("STRING") },
-	{ "xnest-extra-options", 'o', POPT_ARG_STRING, &xnest_options, 0, N_("Extra options for Xnest"), N_("OPTIONS") },
-	{ "background", 'b', POPT_ARG_NONE, &background, 0, N_("Run in background"), NULL },
+static const GOptionEntry xnest_only_options[] = {
+	{ "xnest", 'x', 0, G_OPTION_ARG_STRING, &xnest_binary, N_("Xnest command line"), N_("STRING") },
+	{ "xnest-extra-options", 'o', 0, G_OPTION_ARG_STRING, &xnest_options, N_("Extra options for Xnest"), N_("OPTIONS") },
+	{ "background", 'b', 0, G_OPTION_ARG_NONE, &background, N_("Run in background"), NULL },
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args_remaining, NULL, NULL },
 	{ NULL } 
 };
 
-static const struct poptOption options[] = {
-	{ "xnest", 'x', POPT_ARG_STRING, &xnest_binary, 0, N_("Xnest command line"), N_("STRING") },
-	{ "xnest-extra-options", 'o', POPT_ARG_STRING, &xnest_options, 0, N_("Extra options for Xnest"), N_("OPTIONS") },
-	{ "no-query", 'n', POPT_ARG_NONE, &no_query, 0, N_("Just run Xnest, no query (no chooser)"), NULL },
-	{ "direct", 'd', POPT_ARG_NONE, &do_direct, 0, N_("Do direct query instead of indirect (chooser)"), NULL },
-	{ "broadcast", 'B', POPT_ARG_NONE, &do_broadcast, 0, N_("Run broadcast instead of indirect (chooser)"), NULL },
-	{ "background", 'b', POPT_ARG_NONE, &background, 0, N_("Run in background"), NULL },
-	{ "no-gdm-check", '\0', POPT_ARG_NONE, &no_gdm_check, 0, N_("Don't check for running gdm"), NULL },
+static const GOptionEntry options[] = {
+	{ "xnest", 'x', 0, G_OPTION_ARG_STRING, &xnest_binary, N_("Xnest command line"), N_("STRING") },
+	{ "xnest-extra-options", 'o', 0, G_OPTION_ARG_STRING, &xnest_options, N_("Extra options for Xnest"), N_("OPTIONS") },
+	{ "no-query", 'n', 0, G_OPTION_ARG_NONE, &no_query, N_("Just run Xnest, no query (no chooser)"), NULL },
+	{ "direct", 'd', 0, G_OPTION_ARG_NONE, &do_direct, N_("Do direct query instead of indirect (chooser)"), NULL },
+	{ "broadcast", 'B', 0, G_OPTION_ARG_NONE, &do_broadcast, N_("Run broadcast instead of indirect (chooser)"), NULL },
+	{ "background", 'b', 0, G_OPTION_ARG_NONE, &background, N_("Run in background"), NULL },
+	{ "no-gdm-check", '\0', 0, G_OPTION_ARG_NONE, &no_gdm_check, N_("Don't check for running GDM"), NULL },
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args_remaining, NULL, NULL },
 	{ NULL } 
 };
 
@@ -464,12 +467,10 @@ main (int argc, char *argv[])
 	int display;
 	char *socket;
 	char *pidfile;
-	poptContext ctx;
-	const char **args;
+	GOptionContext *ctx;
 	char *xnest;
 	char **execvec;
 	struct sigaction term;
-	int nextopt;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -477,26 +478,35 @@ main (int argc, char *argv[])
 
 	if (strcmp (base (argv[0]), "gdmXnest") == 0) {
 		gtk_init(&argc, &argv);
-		ctx = poptGetContext(NULL, argc, (const char**)argv, xnest_only_options, 0);
-		while ((nextopt = poptGetNextOpt(ctx)) > 0 || nextopt == POPT_ERROR_BADOPT);
+		ctx = g_option_context_new (_("- Nested gdm login chooser"));
+		g_option_context_add_main_entries (ctx, xnest_only_options, _("main options"));
+		g_option_context_parse (ctx, &argc, &argv, NULL);
+		g_option_context_free (ctx);
 		no_query = TRUE;
 		no_gdm_check = TRUE;
 	} else {
 		gtk_init(&argc, &argv);
-		ctx = poptGetContext(NULL, argc, (const char**)argv, options, 0);
-		while ((nextopt = poptGetNextOpt(ctx)) > 0 || nextopt == POPT_ERROR_BADOPT);
+		ctx = g_option_context_new (_("- Nested gdm login"));
+		g_option_context_add_main_entries (ctx, options, _("main options"));
+		g_option_context_parse (ctx, &argc, &argv, NULL);
+		g_option_context_free (ctx);
 	}
 
-	args = poptGetArgs (ctx);
-	if (args != NULL && args[0] != NULL)
-		host = args[0];
+	if (args_remaining != NULL && args_remaining[0] != NULL)
+		host = args_remaining[0];
+	g_strfreev (args_remaining);
 
+	/* Read config data in bulk */
+	gdmcomm_comm_bulk_start ();
 
 	xdmcp_enabled  = gdm_config_get_bool (GDM_KEY_XDMCP);
 	honor_indirect = gdm_config_get_bool (GDM_KEY_INDIRECT);
-	pidfile        = gdm_config_get_string (GDM_KEY_PID_FILE);
+	pidfile        = GDM_PID_FILE;
 	xnest          = gdm_config_get_string (GDM_KEY_XNEST);
 
+	/* At this point we are done using the socket, so close it */
+	gdmcomm_comm_bulk_stop ();
+
 	/* complex and wonderous way to get the exec vector */
 	execvec = make_us_an_exec_vector (xnest);
 
@@ -658,7 +668,7 @@ main (int argc, char *argv[])
 	xnest_pid = 0;
 
 	socket = g_strdup_printf ("/tmp/.X11-unix/X%d", display);
-	unlink (socket);
+	g_unlink (socket);
 	g_free (socket);
 
 	if (no_query)
